/*
 *  Copyright 2015 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button');

org.ametys.ribbon.control.button.DeletePageButton = function(id, configuration, pluginName)
{
	org.ametys.ribbon.control.button.DeletePageButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.ribbon.control.button.DeletePageButton, org.ametys.ribbon.control.button.ModifiablePageActionButton, {});


org.ametys.ribbon.control.button.DeletePageButton.prototype.onMessage = function(messages)
{
	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));
			
			if (target != null)
			{
				if (target.getParameters().isModifiable != 'true')
				{
					var text = this._getTooltip(this.getConfiguration()["no-modifiable-description"]);
					this.updateTooltip(text, null, null);
					
					this.getElements().each(function(element) {element.disable();});
				}
				else 
				{
					this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
					this.updateTooltip(this.getConfiguration()["default-description"]);
					
					org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: [target.getParameters().id]});
				}
			}
			else
			{
				var text = this._getTooltip(this.getConfiguration()["no-selection-description"]);
				this.updateTooltip(text, null, null);
				this.getElements().each(function(element) {element.disable();});
			}
		}
	}
}

org.ametys.ribbon.control.button.DeletePageButton.prototype.configurationError = function(response)
{
    this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
    this.updateTooltip(this.getConfiguration()['default-description']);
}

org.ametys.ribbon.control.button.DeletePageButton.prototype.configurationChanged = function(params)
{
    this.getElements().each(function(element) { element.stopRefreshing(); });
    
    if (params['has-right'] == 'false')
    {
        this.getElements().each(function(element) {element.disable();});
        var text = this._getTooltip(this.getConfiguration()["no-right-description"]);
		this.updateTooltip(text, null, null);
    }
    else
    {
        this.getElements().each(function(element) {element.enable();});
        this.updateTooltip(this.getConfiguration()['default-description'], null, null);
    }
}
