/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button');

/**
 * Create the control instance.
 * @class This class provides an {@link org.ametys.ribbon.control.button.ActionButton} which is enabled if the selection is an editable resource explorer node
 * @extends {@link org.ametys.ribbon.control.button.ActionButton}
 * @param {String} id The unique identifier
 * @param {Map<id, value>} configuration The configuration of the control instance.
 * @param {String} pluginName The name of the declaring plugin
 * @constructor
 */
org.ametys.ribbon.control.button.ImagesResourceButton = function(id, configuration, pluginName)
{
	org.ametys.ribbon.control.button.ImagesResourceButton.superclass.constructor.call(this, id, configuration, pluginName);
}

Ext.extend(org.ametys.ribbon.control.button.ImagesResourceButton, org.ametys.ribbon.control.button.ActionSelectionButton, {});

org.ametys.ribbon.control.button.ImagesResourceButton.prototype._images = [];
org.ametys.ribbon.control.button.ImagesResourceButton.prototype._currentNodeId;

org.ametys.ribbon.control.button.ImagesResourceButton.prototype.onMessage = function (messages)
{
	for (var i=messages.length-1; i >=0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));
			
			if (target != null)
			{
				this._images = [];
				this._currentNodeId = target.getParameters().id;
				
				if (this._isDisabled (target))
				{
					var text = this._getTooltip(this.getConfiguration()["no-selection-description"]);
					this.updateTooltip(text, null, null);
					this.getElements().each(function(element) {element.disable();});
				}
				else
				{
					this._refreshBtn();
				}
			}
			else
			{
				this._currentNodeId = null;
				this._images = [];
				var text = this._getTooltip(this.getConfiguration()["no-selection-description"]);
				this.updateTooltip(text, null, null);
				this.getElements().each(function(element) {element.disable();});
			}
		}
		else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_CREATED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function (target) {return target.getType() == 'resource'});
			
			if (target != null && this._currentNodeId != null && messages[i].getType().getParameters().parentID == this._currentNodeId)
			{
				this._refreshBtn();
			}
		}
	}
}

org.ametys.ribbon.control.button.ImagesResourceButton.prototype._refreshBtn = function ()
{
	this.getElements().each(
			function(element) 
			{ 
				element.setDisabled(true);
				element.toggle(false);
				element.refreshing();
			});
	
	var serverMessage = new org.ametys.servercomm.ServerMessage(this.getPluginName(), '/resource-collection/images.xml', {'id': this._currentNodeId}, org.ametys.servercomm.ServerComm.PRIORITY_MAJOR, this._refreshBtnCb, this, [this._currentNodeId]);
	org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
}

org.ametys.ribbon.control.button.ImagesResourceButton.prototype._refreshBtnCb = function (response, args)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });
	
	if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key='PLUGINS_WEB_EXPLORER_GET_IMAGE_ERROR'/>", response, 'org.ametys.ribbon.control.button.ImagesResourceButton'))
    {
    	throw "org.ametys.ribbon.control.button.ImagesResourceButton request failed";
    }
	
	var images = response.selectNodes('images/file');
    if (images.length == 0)
    {
    	var text = this._getTooltip(this.getConfiguration()["no-image-selection"]);
		this.updateTooltip(text, null, null);
		this.getElements().each(function(element) {element.disable();});
    }
    else
    {
    	for (var j=0; j < images.length; j++)
    	{
    		this._images.push(images[j][org.ametys.servercomm.ServerComm.xmlTextContent]);
    	}
    	
    	var text = this._getTooltip(null);
		this.updateTooltip(text, null, null);
		this.getElements().each(function(element) {element.enable();});
    }
}

org.ametys.ribbon.control.button.ImagesResourceButton.prototype._isDisabled = function (target)
{
	return target.getParameters()['applicationId'] != this.getConfiguration()['application-id'];
}

org.ametys.ribbon.control.button.ImagesResourceButton.prototype._getTooltip = function (addedText)
{
	var tooltip = this.getConfiguration()["default-description"];
	if (addedText != null && addedText.length > 0)
	{
		tooltip += "<br/><br/>";
		
		tooltip += addedText;
	}
	return tooltip;
}


