/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button.');

org.ametys.ribbon.control.button.SmartTagButton = function(id, configuration, pluginName) 
{
	org.ametys.ribbon.control.button.SmartTagButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.ribbon.control.button.SmartTagButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property {Map<String, String>} _lastParams The params received on the last refresh
 */
org.ametys.ribbon.control.button.SmartTagButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentContentsId The content id currently selected or null. 
 */
org.ametys.ribbon.control.button.SmartTagButton.prototype._currentContentsId;
/**
 * @private
 * @property {String[]} _currentContentsId The content id currently selected or null. 
 */
org.ametys.ribbon.control.button.SmartTagButton.prototype._currentPagesId;

org.ametys.ribbon.control.button.SmartTagButton.prototype._areTheSame = function (tab1, tab2)
{
	if (tab1 == tab2)
	{
		return true;
	}

	if (tab1 == null || tab2 == null || tab1.length != tab2.length)
	{
		return false;
	}	
	
	for (var i=0; i < tab1.length; i++)
	{
		if (tab2.indexOf(tab1[i]) == -1)
		{
			return false;
		}
	}
	
	return true;
}

org.ametys.ribbon.control.button.SmartTagButton.prototype.onMessage = function (messages)
{
	// SELECTION
	for (var i=messages.length - 1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var allContentTargets = this._convertToId(org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkContentTarget()));
			var allPageTargets = this._convertToId(org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkPageTarget()));
			
			// No content and no page selected
			if (allContentTargets.length == 0 && allPageTargets.length == 0)
			{
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["no-selection-description"]);
				
				this._currentContentsId = [];
				this._currentPagesId = [];
			}
			else if (allPageTargets.length > 1) // page multiselection is disabled
			{
				this._currentContentsId = [];
				this._currentPagesId = [];
				
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["page-multiselection-disabled-description"]);
			}
			else if (allPageTargets.length == 0 && allContentTargets.length > 1) // content multiselection is disabled
			{
				this._currentContentsId = [];
				this._currentPagesId = [];
				
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["content-multiselection-disabled-description"]);
			}
			else
			{
				if (!this._areTheSame(this._currentContentsId, allContentTargets) || !this._areTheSame(this._currentPagesId, allPageTargets))
				{
					this._currentContentsId = [];
					this._currentPagesId = [];
					
					this.getElements().each(
						function(element) 
						{ 
							element.setDisabled(true);
							element.toggle(false);
							element.refreshing();
						}
					);

					this.updateTooltip(this.getConfiguration()["refresh-description"]);
					org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {contentIds: allContentTargets.length == 1 ? allContentTargets : [], pageIds: allPageTargets});
				}
			}
		}
	}
}

org.ametys.ribbon.control.button.SmartTagButton.prototype._convertToId = function (targets)
{
	var targetsIds = [];
	for (var i = 0; i < targets.length; i++)
	{
		targetsIds.push(targets[i].getParameters()['id']);
	}
	return targetsIds;
}
org.ametys.ribbon.control.button.SmartTagButton.prototype.checkContentTarget = function ()
{
	return function (target) { return target.getType() == 'content'; };
}
org.ametys.ribbon.control.button.SmartTagButton.prototype.checkPageTarget = function ()
{
	return function (target) { return target.getType() == 'page'; };
}

org.ametys.ribbon.control.button.SmartTagButton.prototype._getContentIds = function (params)
{
	var contentIds = [];
	var count = parseInt(params["allright-content-size"]);
	if (count > 0)
	{
		for (var i=0; i < count; i++)
		{
			contentIds.push(params["allright-content-id-" + i])
		}
	}
	return contentIds;
}

org.ametys.ribbon.control.button.SmartTagButton.prototype._getPageIds = function (params)
{
	var pageIds = [];
	var count = parseInt(params["allright-page-size"]);
	if (count > 0)
	{
		for (var i=0; i < count; i++)
		{
			pageIds.push(params["allright-page-id-" + i])
		}
	}
	return pageIds;
}

org.ametys.ribbon.control.button.SmartTagButton.prototype._handlingMultiple = function(tooltip, paramPrefix, params)
{
	var count = parseInt(params[paramPrefix + "-size"]);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()[paramPrefix + "-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params[paramPrefix + "-" + i];
		}
		tooltip += this.getConfiguration()[paramPrefix + "-end-description"];
	}
	
	return tooltip;
}

org.ametys.ribbon.control.button.SmartTagButton.prototype.configurationError = function(response)
{
	this.getElements().each(
		function(element) 
		{ 
			element.setDisabled(true);
			element.stopRefreshing();
		}
	);
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.ribbon.control.button.SmartTagButton.prototype.configurationChanged = function(params)
{
	this._currentContentsId = this._getContentIds (params);
	this._currentPagesId = this._getPageIds(params);
	
	this.getElements().each(function(element) { element.stopRefreshing(); });

	var tooltip = this._handlingMultiple("", "allright-page", params);
	tooltip = this._handlingMultiple(tooltip, "allright-content", params);
	
	var hasOneElmt = tooltip.length > 0; 
	if (!hasOneElmt)
	{
		tooltip += this.getConfiguration()['error-description'];
	}
	
	tooltip = this._handlingMultiple(tooltip, "noright-page", params);
	tooltip = this._handlingMultiple(tooltip, "noright-content", params);
	tooltip = this._handlingMultiple(tooltip, "locked-content", params);
	tooltip = this._handlingMultiple(tooltip, "nomodifiable-content", params);
	tooltip = this._handlingMultiple(tooltip, "nomodifiable-page", params);
	
	this.getElements().each(
			function(element) 
			{ 
				element.allowDepress = false;
				element.setDisabled(!hasOneElmt);
			}
		);
	
	this.updateTooltip(tooltip);
	this._lastParams = params;
}
