/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button');

/**
 * Create the control instance.
 * @param {String} id The unique identifier
 * @param {Map<id, value>} configuration The configuration of the control instance.
 * @param {String} pluginName The name of the declaring plugin
 * @class This class implements the behavior for an action.
 * @extends org.ametys.ribbon.control.RibbonControl
 */
org.ametys.ribbon.control.button.TagActionButton = function(id, configuration, pluginName)
{
	org.ametys.ribbon.control.button.TagActionButton.superclass.constructor.call(this, id, configuration, pluginName);
}

Ext.extend(org.ametys.ribbon.control.button.TagActionButton, org.ametys.ribbon.control.button.ActionButton, {});

org.ametys.ribbon.control.button.TagActionButton.prototype.createControl = function(colspan)
{
	var elmt = org.ametys.ribbon.control.button.TagActionButton.superclass.createControl.call(this, colspan);
	elmt.disable();
	return elmt;
}

/**
 * Creates the toggle button
 * @param {integer} colspan The configured colspan. Must be transmited to the control.
 * @return {Ext.form.field} The toggle button created. 
 */
org.ametys.ribbon.control.button.TagActionButton.prototype.onMessage = function(messages)
{
	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "tag";});
			
			if (target != null)
			{
				var types = this.getConfiguration()['type'].split('|');
				for (var i=0; i < types.length; i++)
				{
					if (target.getParameters().type == types[i])
					{
						this.getElements().each(function(element) { element.enable(); })
						return;
					}
				}
			}
			
			this.getElements().each(function(element) { element.disable(); })
		}
	}
}
