/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button');

org.ametys.ribbon.control.button.ToggleRobotsActionButton = function(id, configuration, pluginName) 
{
	org.ametys.ribbon.control.button.ToggleRobotsActionButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.ribbon.control.button.ToggleRobotsActionButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property {Map<String, String>} _lastParams The parameters received on the last refresh
 */
org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentPagesId The pages id currently selected or null. 
 */
org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._currentPagesId;
/**
 * @private
 * @property {String[]} _currentIncludedPagesId The page id currently included in seo files 
 */
org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._currentIncludedPagesId;
/**
 * @private
 * @property {String[]} _currentIncludedPagesId The page id currently excluded from seo files 
 */
org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._currentExcludedPagesId;

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._areTheSame = function (tab1, tab2)
{
	if (tab1 == tab2)
	{
		return true;
	}

	if (tab1 == null || tab2 == null || tab1.length != tab2.length)
	{
		return false;
	}	
	
	for (var i=0; i < tab1.length; i++)
	{
		if (tab2.indexOf(tab1[i]) == -1)
		{
			return false;
		}
	}
	
	return true;
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._isInSelection = function (tab1, tab2)
{
	if (tab1 == tab2)
	{
		return true;
	}
	
	if (tab1 == null || tab1.length == 0 || tab2 == null || tab2.length == 0)
	{
		return false;
	}
	
	for (var i=0; i < tab2.length; i++)
	{
		if (tab1.indexOf(tab1[i]) != -1)
		{
			return true;
		}
	}
	return false;
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._doReload = function(allTargets)
{
	this._currentPagesId = allTargets;
	this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["refresh-description"]);
	org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: allTargets});
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype.onMessage = function (messages)
{
	for (var i=messages.length - 1; i >= 0; i--)
	{
		// SELECTION
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var allTargets = this._convertToId(org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget()));
			
			// No page selected
			if (allTargets.length == 0)
			{
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["default-description"]);
				
				this._currentPagesId = [];
			}
			else
			{
				if (!this._areTheSame(this._currentPagesId, allTargets))
				{
					this._doReload(allTargets);
				}
			}
		}
		else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_METADATACHANGED
		        || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_MINORMETADATACHANGED)
		{
			var targets = messages[i].getTargets();
			var allTargets = this._convertToId(org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget()));
			
			if (allTargets.length != 0 && this._isInSelection (this._currentPagesId, allTargets))
			{
				this._doReload(allTargets);
			}
		}
	}
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._convertToId = function (targets)
{
	var targetsIds = [];
	for (var i = 0; i < targets.length; i++)
	{
		targetsIds.push(targets[i].getParameters()['id']);
	}
	return targetsIds;
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype.checkTarget = function ()
{
	return function (target) { return target.getType() == 'page' && target.getParameters().isModifiable == 'true'};
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._getTooltip = function (tooltip, params)
{
	this._currentIncludedPagesId = [];
	this._currentExcludedPagesId = [];
	
	var count = parseInt(params['page-included-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["page-included-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["page-included-description-" + i];
			this._currentIncludedPagesId.push(params["page-included-" + i]);
		}
		tooltip += this.getConfiguration()["page-included-end-description"];
	}
	
	var count = parseInt(params['page-excluded-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["page-excluded-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["page-excluded-description-" + i];
			this._currentExcludedPagesId.push(params["page-excluded-" + i]);
		}
		tooltip += this.getConfiguration()["page-excluded-end-description"];
	}
	
	var count = parseInt(params['page-parent-excluded-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["page-parent-excluded-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["page-parent-excluded-description-" + i];
			this._currentExcludedPagesId.push(params["page-parent-excluded-" + i]);
		}
		tooltip += this.getConfiguration()["page-parent-excluded-end-description"];
	}
	
	if (parseInt(params['page-noright-size']) > 0)
	{
		tooltip += "<br/><br/>";
		tooltip += this.getConfiguration()['no-right-description'];
	}
	else if (parseInt(params['page-excluded-size']) > 0)
	{
		tooltip += "<br/><br/>";
		tooltip += this.getConfiguration()["page-include-description"];
	}
	else if (parseInt(params['page-included-size']) > 0)
	{
		tooltip += "<br/><br/>";
		tooltip += this.getConfiguration()["page-exclude-description"];
	}
	else if (parseInt(params['page-parent-excluded-size']) > 0)
	{
		tooltip += "<br/><br/>";
		tooltip += this.getConfiguration()["page-parent-exclude-description"];
	}
		
	return tooltip;
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._getIcon = function(params)
{
	var nbIncludedPages = parseInt(params['page-included-size']);
	var nbExcludedPages = parseInt(params['page-excluded-size']);
	var nbParentExcludedPages = parseInt(params['page-parent-excluded-size']);
	
	if (nbIncludedPages > 0 && nbExcludedPages == 0 && nbParentExcludedPages == 0)
	{
		var iconSmall = context.contextPath + this.getConfiguration()['icon-small'];
		var iconLarge = context.contextPath + this.getConfiguration()['icon-medium'];
		this.getElements().each(
			function (element) 
			{
				element.iconSmall = iconSmall;
				element.iconLarge = iconLarge;
			}
		);
	}
	
	if (nbExcludedPages > 0 || nbParentExcludedPages > 0)
	{
		var iconSmall = context.contextPath + this.getConfiguration()['excluded-icon-small'];
		var iconLarge = context.contextPath + this.getConfiguration()['excluded-icon-medium'];
		this.getElements().each(
			function (element) 
			{
				element.iconSmall = iconSmall;
				element.iconLarge = iconLarge;
			}
		);
	}
}
org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype._isDisabled = function (params)
{
	var nbIncludedPages = parseInt(params['page-included-size']);
	var nbExcludedPages = parseInt(params['page-excluded-size']);
	var nbParentExcludedPages = parseInt(params['page-parent-excluded-size']);
	var nbNorightsPages = parseInt(params['page-noright-size']);
	
	return (nbIncludedPages == 0 && nbExcludedPages == 0 && nbParentExcludedPages > 0) || nbNorightsPages > 0;
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.ribbon.control.button.ToggleRobotsActionButton.prototype.configurationChanged = function(params)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });
	
	var tooltip = this._getTooltip ("", params);
	
	var hasOneContent = tooltip.length > 0; 
	if (!hasOneContent)
	{
		tooltip += this.getConfiguration()['no-selection-description'];
	}
	
	// Icon
	this._getIcon(params);
	
	this.getElements().each(function(element) { element.updateIcon(); });
	this.updateTooltip(tooltip, this.getElements().get(0).iconLarge, null);
	
	var disabled = this._isDisabled(params);
	var nbExcludedPages = parseInt(params['page-excluded-size']);
	var nbParentExcludedPages = parseInt(params['page-parent-excluded-size']);
	this.getElements().each(function(element) { element.toggle(nbExcludedPages > 0 || nbParentExcludedPages > 0); });
	this.getElements().each(function(element) { element.setDisabled(!hasOneContent || disabled); });
	
	this._lastParams = params;
}
