/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.web.content');

org.ametys.web.content.DeleteButton = function(id, configuration, pluginName) 
{
	org.ametys.web.content.DeleteButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.web.content.DeleteButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property _lastParams The params received on the last refresh
 */
org.ametys.web.content.DeleteButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentContentsId The content id currently selected or null. 
 */
org.ametys.web.content.DeleteButton.prototype._currentContentsId;

/**
 * @private
 * @property {String[]} _currentOrphanContentsId The contents id currently selected and orphans. 
 */
org.ametys.web.content.DeleteButton.prototype._currentOrphanContentsId;

/**
 * @private
 * @property {String[]} _currentReferencedContentsId The contents id currently selected and referenced. 
 */
org.ametys.web.content.DeleteButton.prototype._currentReferencedContentsId;


org.ametys.web.content.DeleteButton.prototype.onMessage = function (messages)
{
	// SELECTION
	for (var i=messages.length - 1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var allTargetsObject = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
			var allTargets = this._convertToId(allTargetsObject);
			
			// No content selected
			if (allTargets.length == 0)
			{
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["no-content"]);
				
				this._currentContentsId = [];
			}
			else
			{
				if (!this._areTheSame(this._currentContentsId, allTargets))
				{
					this._currentContentsId = allTargets;
					this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
					this.updateTooltip(this.getConfiguration()["default-description"]);
					org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: allTargets});
				}
			}
			
			return;
		}
	}
}

org.ametys.web.content.DeleteButton.prototype._areTheSame = function (tab1, tab2)
{
	if (tab1 == tab2)
	{
		return true;
	}

	if (tab1 == null || tab2 == null || tab1.length != tab2.length)
	{
		return false;
	}	
	
	for (var i=0; i < tab1.length; i++)
	{
		if (tab2.indexOf(tab1[i]) == -1)
		{
			return false;
		}
	}
	
	return true;
}

org.ametys.web.content.DeleteButton.prototype._isInSelection = function (tab, elmt)
{
	if (tab == null || tab.length == 0)
	{
		return false;
	}	
	
	for (var i=0; i < tab.length; i++)
	{
		if (tab[i] == elmt)
		{
			return true;
		}
	}
	
	return false;
	
}

org.ametys.web.content.DeleteButton.prototype._convertToId = function (targets)
{
	var targetsIds = [];
	for (var i = 0; i < targets.length; i++)
	{
		targetsIds.push(targets[i].getParameters()['id']);
	}
	return targetsIds;
}
org.ametys.web.content.DeleteButton.prototype.checkTarget = function ()
{
	return function (target) { return target.getType() == 'content'; };
}

org.ametys.web.content.DeleteButton.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.web.content.DeleteButton.prototype.configurationChanged = function(params)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });
	
	var tooltip = this._getTooltip("", params);
	
	var hasOneContent = tooltip.length > 0; 
	if (!hasOneContent)
	{
		tooltip += this.getConfiguration()['no-content'];
	}
	
	var disabledState = this._isDisabled(params);
	this.getElements().each(function(element) { element.setDisabled(disabledState); });
	this.updateTooltip(tooltip);
	
	this._lastParams = params;
}

org.ametys.web.content.DeleteButton.prototype._isDisabled = function(params)
{
	var nbOrphans = parseInt(params['orphan-content-size']);
	if (nbOrphans == 0)
	{
		return true;
	}
	
	return false;
}

org.ametys.web.content.DeleteButton.prototype._getTooltip = function(tooltip, params)
{
	this._currentOrphanContentsId = [];
	this._currentReferencedContentsId = [];
	
	var count = parseInt(params['orphan-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["orphan-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["orphan-content-description-" + i];
			this._currentOrphanContentsId.push(params["orphan-content-" + i]);
		}
		tooltip += this.getConfiguration()["orphan-content-end-description"];
	}
	
	var count = parseInt(params['unauthorized-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["unauthorized-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["unauthorized-content-description-" + i];
			this._currentReferencedContentsId.push(params["unauthorized-content-" + i]);
		}
		tooltip += this.getConfiguration()["unauthorized-content-end-description"];
	}
	
	var count = parseInt(params['locked-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["locked-content-start-description"];
		for (var i=0; i < count; i++)
		{
			tooltip += "<br/>";
			tooltip += params["locked-content-description-" + i];
			this._currentReferencedContentsId.push(params["locked-content-" + i]);
		}
		tooltip += this.getConfiguration()["locked-content-end-description"];
	}
	
	var count = parseInt(params['referenced-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["referenced-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["referenced-content-description-" + i];
			this._currentReferencedContentsId.push(params["referenced-content-" + i]);
		}
		tooltip += this.getConfiguration()["referenced-content-end-description"];
	}
	
	
	
	return tooltip;
}


