/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.web.menu');

org.ametys.web.menu.SetContentViewMenu = function(id, configuration, pluginName) 
{
	org.ametys.web.menu.SetContentViewMenu.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.web.menu.SetContentViewMenu, org.ametys.cms.ribbon.button.Menu, {});

org.ametys.web.menu.SetContentViewMenu.prototype._pageTarget;

org.ametys.web.menu.SetContentViewMenu.prototype.onMessage = function(messages)
{
	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			
			var ziTargets = null;
			this._pageTarget = null;
			
			var pageTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) {return target.getType() == 'page'} );
		    if (pageTarget != null && pageTarget.getParameters()['type'] == 'container')
			{
			    ziTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(pageTarget.getSubTargets(), function(target) {return target.getType() == 'zoneitem' && target.getParameters().type == 'content';} );
			}
			
            if (ziTargets == null || ziTargets.length == 0)
            {
                this.updateTooltip(this._getTooltip(this.getConfiguration()['no-selection-description']), null, null);
                this.getElements().each(function(element) { element.disable(); })
            }
            else if (ziTargets.length > 1)
            {
                this.updateTooltip(this._getTooltip(this.getConfiguration()['multi-selection-description']), null, null);
                this.getElements().each(function(element) { element.disable(); })
            }
            else
            {
                this._pageTarget = pageTarget;
                var zoneItemTarget = ziTargets[0];
                
                if (zoneItemTarget.getParameters().isModifiable != 'true')
                {
                    this.updateTooltip(this._getTooltip(this.getConfiguration()['no-modifiable-description']), null, null);
                    
                    this.getElements().each(function(element) { element.disable(); });
                }
                else if (!this._hasRight(pageTarget))
                {
                    this.updateTooltip(this._getTooltip(this.getConfiguration()['no-right-description']), null, null);
                    this.getElements().each(function(element) { element.disable(); });
                }
                else
                {
                    if (this.getConfiguration()['selection-listener'])
                    {
                        eval(this.getConfiguration()['selection-listener'] + "(this, zoneItemTarget);");
                    }
                    else
                    {
                        this.updateTooltip(this._getTooltip(), null, null);
                        this.getElements().each(function(element) { element.enable(); });
                    }
                }
            }
		}
	}
}

org.ametys.web.menu.SetContentViewMenu.prototype.getPageTarget = function()
{
    return this._pageTarget;
}

org.ametys.web.menu.SetContentViewMenu.prototype._getGalleryButtonCfg = function(index, groupElement)
{
    var cfg = org.ametys.web.menu.SetContentViewMenu.superclass._getGalleryButtonCfg.call(this, index, groupElement);
    
    var prefix = "gallery-" + index;
    
    var newConfig = Ext.applyIf(cfg, {
        contentTypeId: this.getConfiguration()[prefix + "-contentTypeId"],
        metadataSetName: this.getConfiguration()[prefix + "-metadataSetName"]
    });
    
    return newConfig;
}

org.ametys.web.menu.SetContentViewMenu.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()['default-description']);
}

org.ametys.web.menu.SetContentViewMenu.prototype.configurationChanged = function(params)
{
	if (this.getConfiguration()['configuration-changed-listener'])
	{
		eval(this.getConfiguration()['configuration-changed-listener'] + "(this, params);");
	}
}

/**
 * Checks the user right on current target
 * @return true if the user has right to do the action
 * @private
 */
org.ametys.web.menu.SetContentViewMenu.prototype._hasRight = function(target, id)
{
	id = id || 'right-id';
	
	if (!this.getConfiguration()[id])
	{
		// No right is needed
		return true;
	}
	
	var rights = target.getParameters().rights;
	var neededRights = this.getConfiguration()[id].split('|');
	for (var i=0; neededRights != null && i < neededRights.length; i++)
	{
		if (rights.indexOf(neededRights[i]) != -1)
		{
			return true;
		}
	}
	
	return false;
}

org.ametys.web.menu.SetContentViewMenu.prototype._getTooltip = function(addedText)
{
	var tooltip = this.getConfiguration()['default-description'];
	if (addedText != null && addedText.length > 0)
	{
		tooltip += "<br/><br/>";
		
		tooltip += addedText;
	}
	return tooltip;
}

org.ametys.web.menu.SetContentViewMenuSelectionListener = function(menu, zoneItemTarget)
{
    menu.updateTooltip(menu._getTooltip(), null, null);
    
    var metadataSetName = zoneItemTarget.getParameters().metadataSetName;
    
    var contentTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(zoneItemTarget.getSubTargets(), function(target) {return target.getType() == 'content'} );
    var contentType = contentTarget.getParameters().type;
    
    menu.getElements().each(function(element) {
        element.enable();
		
        for (var itemId in element._galleryItems)
        {
            var item = element._galleryItems[itemId];
			
            item.setVisible(item.contentTypeId == contentType);
            item.toggle(false);
            
            if (item.metadataSetName == metadataSetName)
            {
                element.iconSmall = item.iconSmall;
                element.iconLarge = item.iconLarge;
                element.updateIcon();
                
                item.toggle(true);
            }
            
            if (item.groupEl != null && typeof item.groupEl.setVisible == 'function')
            {
                item.groupEl.setVisible(item.contentTypeId == contentType);
            }
        }
    });
}
