/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.web.tool.content');

/**
 * Do not call this constructor. Declared to and used by the {@see org.ametys.uitool.ToolsManager}
 * @constructor
 * @extends org.ametys.uitool.tool.ToolFactory
 * @class The factory to creates content tools.
 * @param {String} id The unique id of the factory
 * @param {Map} config 
 * @param {String} pluginName The name of the plugin where the factory was declared
 */
org.ametys.web.tool.content.WebContentToolFactory = function(id, config, pluginName) 
{
	org.ametys.web.tool.content.WebContentToolFactory.superclass.constructor.call(this, id, config, pluginName);
};

Ext.extend(org.ametys.web.tool.content.WebContentToolFactory, org.ametys.uitool.tool.ToolFactory, {});

/**
 * The prefix used to generate the tool id.
 * @type {String} 
 * @private
 * @constant
 */
org.ametys.web.tool.content.WebContentToolFactory._toolIdPrefix = "content-";

/**
 * Creates a tool of type {@see org.ametys.cms.tool.content.ContentTool} or activate the existing one.
 * There is one tool per content.
 * @param {Map} params The params used by the factory are : 'id' that is the content id. Params are transmited the the content tool : {@see org.ametys.cms.tool.content.ContentTool#setParams}
 * @return {Ext.org.ametys.uitool.tool.Tool} The tool to open.
 */
org.ametys.web.tool.content.WebContentToolFactory.prototype.openTool = function (params)
{
	if (org.ametys.uitool.ToolsManager.getInstance().getTool(org.ametys.web.tool.content.WebContentToolFactory._toolIdPrefix + params.id) == null)
	{
		// Tool does not exists => Tool creation
		this._config['title'] = "...";
		var tool = new org.ametys.web.tool.content.WebContentTool (org.ametys.web.tool.content.WebContentToolFactory._toolIdPrefix + params.id, this.getPluginName());
		
		tool.setConfiguration(this._config);
		tool.setFactory (this);
		
		org.ametys.uitool.ToolsManager.getInstance().addUITool (tool, this._config['default-location']);
	}
	else
	{
		// Tool does already exists => Tool activation
		tool = org.ametys.uitool.ToolsManager.getInstance().getTool(org.ametys.web.tool.content.WebContentToolFactory._toolIdPrefix + params.id);
	}

	tool.setParams(params);
	tool.activate();

	return tool;
}

/**
 * Destroy the tool.
 * @param {org.ametys.uitool.tool.Tool} tool The tool to close.
 */
org.ametys.web.tool.content.WebContentToolFactory.prototype.closeTool = function (tool)
{
	org.ametys.uitool.ToolsManager.getInstance().removeUITool (tool);
}
