/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.web.tool.pageaccess.control');

org.ametys.web.tool.pageaccess.control.PageAccessActionButton = function(id, configuration, pluginName)
{
	org.ametys.web.tool.pageaccess.control.PageAccessActionButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.web.tool.pageaccess.control.PageAccessActionButton, org.ametys.ribbon.control.button.RightActionSelectionButton, {});

org.ametys.web.tool.pageaccess.control.PageAccessActionButton.prototype.onMessage = function(messages)
{
    for (var i=0; i < messages.length; i++)
    {
        if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
        {
            var targets = messages[i].getTargets();
            var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));
            
            if (target != null)
            {
                if (!this._hasRight(target))
                {
                    var text = this._getTooltip(this.getConfiguration()["no-right-description"]);
                    this.updateTooltip(text, null, null);
                    
                    this.getElements().each(function(element) {element.disable();});
                }
                else
                {
                    var params = {targets: [target.getParameters().id]};
                    
                    // Set the toggled state.
                    org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, params);
                }
            }
            else
            {
                var text = this._getTooltip(this.getConfiguration()["no-selection-description"]);
                this.updateTooltip(text, null, null);
                this.getElements().each(function(element) {element.disable();});
            }
        }
    }
}

org.ametys.web.tool.pageaccess.control.PageAccessActionButton.prototype.configurationError = function(response)
{
    this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
    this.updateTooltip(this.getConfiguration()['default-description']);
}

org.ametys.web.tool.pageaccess.control.PageAccessActionButton.prototype.configurationChanged = function(params)
{
    this.getElements().each(function(element) { element.stopRefreshing(); });
    
    var allowAnyConnectedUserCount = parseInt(params['allow-any-connected-user-count']);
    var inheritedCount = parseInt(params['inherited-rectriction-count']);
    
    if (allowAnyConnectedUserCount > 0 && inheritedCount == 0 && this.getConfiguration()['disable-on-allow-any-user'] == 'true')
    {
        // If any connected user is allowed on the page itself (i.e. if not inherited), disable the button.
        this.getElements().each(function(element) {element.disable();});
        this.updateTooltip(this.getConfiguration()['any-connected-user-description'], null, null);
    }
    else
    {
        this.getElements().each(function(element) {element.enable();});
        this.updateTooltip(this.getConfiguration()['default-description'], null, null);
    }
}
