/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.web.tool.pageaccess.control');

org.ametys.web.tool.pageaccess.control.PageAccessButton = function(id, configuration, pluginName) 
{
	org.ametys.web.tool.pageaccess.control.PageAccessButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.web.tool.pageaccess.control.PageAccessButton, org.ametys.ribbon.control.button.ToolButton, {});

/**
 * @private
 * @property {Map<String, String>} _lastParams The parameters received on the last refresh
 */
org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentPagesId The pages id currently selected or null. 
 */
org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype._currentPagesId;

org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype._areTheSame = function (tab1, tab2)
{
	if (tab1 == tab2)
	{
		return true;
	}

	if (tab1 == null || tab2 == null || tab1.length != tab2.length)
	{
		return false;
	}	
	
	for (var i=0; i < tab1.length; i++)
	{
		if (tab2.indexOf(tab1[i]) == -1)
		{
			return false;
		}
	}
	
	return true;
}

org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype._doReload = function()
{
	if (this._currentPagesId.length == 1)
	{
		this.getElements().each(
			function(element)
			{
				element.setDisabled(true);
				element.refreshing();
			}
		);
		this.updateTooltip(this.getConfiguration()["refresh-description"]);
		org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: this._currentPagesId});
	}
}
org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype.onMessage = function (messages)
{
	// SELECTION
	for (var i=messages.length - 1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var allTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
			var allTargetsId = this._convertToId(allTargets);
			
			// No page selected
			if (allTargets.length == 0)
			{
				this._currentPagesId = [];
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["default-description"]);
				break;
			}
			// A single target selected or multiselection is enabled
			else if (allTargets.length == 1)
			{
				if (!this._areTheSame(this._currentPagesId, allTargetsId))
				{
					this._currentPagesId = allTargetsId;
					
					// Only test modifiable if the target is a page.
					if (allTargets[0].getType() == 'page' && allTargets[0].getParameters().isModifiable != 'true')
					{
						var iconSmall = context.contextPath + this.getConfiguration()['icon-small'];
						var iconLarge = context.contextPath + this.getConfiguration()['icon-medium'];
						
						this.getElements().each(
								function (element) 
								{
									element.iconSmall = iconSmall;
									element.iconLarge = iconLarge;
								}
							);
						
						this.getElements().each(function(element) { element.updateIcon(); });
						this.updateTooltip(this.getConfiguration()["no-modifiable-description"], iconLarge, null);
						
						this.getElements().each(function(element) {element.disable();});
					}
					else
					{
					    // Case of a modifiable page or a sitemap.
						this.getElements().each(function(element) { element.setDisabled(true); element.refreshing(); });
						this.updateTooltip(this.getConfiguration()["refresh-description"]);
						org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: this._currentPagesId});
					}
					
					break;
				}
			}
			// Too many pages selected
			else
			{
				this._currentPagesId = [];
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["multiselection-disabled-description"]);
				break;
			}
		}
		else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_MINORMETADATACHANGED)
		{
			var targets = messages[i].getTargets();
			var allTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
			var allTargetsId = this._convertToId(allTargets);
			
			// No page selected
			if (allTargets.length == 0)
			{
				this._currentPagesId = [];
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["default-description"]);
				break;
			}
			// A single target selected or multiselection is enabled
			else if (allTargets.length == 1)
			{
				this._currentPagesId = allTargetsId;
				
				if (allTargets[0].getParameters().isModifiable != 'true')
				{
					var iconSmall = context.contextPath + this.getConfiguration()['icon-small'];
					var iconLarge = context.contextPath + this.getConfiguration()['icon-medium'];
					
					this.getElements().each(
							function (element) 
							{
								element.iconSmall = iconSmall;
								element.iconLarge = iconLarge;
							}
						);
					
					this.getElements().each(function(element) { element.updateIcon(); });
					this.updateTooltip(this.getConfiguration()["no-modifiable-description"], iconLarge, null);
					
					this.getElements().each(function(element) {element.disable();});
				}
				else
				{
					this.getElements().each(function(element) { element.setDisabled(true); element.refreshing(); });
					this.updateTooltip(this.getConfiguration()["refresh-description"]);
					org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: this._currentPagesId});
				}
				
				break;
			}
			// Too many pages selected
			else
			{
				this._currentPagesId = [];
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["multiselection-disabled-description"]);
				break;
			}
		}
	}
}

org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype._convertToId = function (targets)
{
	var targetsIds = [];
	for (var i = 0; i < targets.length; i++)
	{
		targetsIds.push(targets[i].getParameters()['id']);
	}
	return targetsIds;
}
org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype.checkTarget = function ()
{
	return function (target) { return target.getType() == 'page' || target.getType() == 'sitemap'; };
}

org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype.configurationChanged = function(params)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });

	this._getIcon(params);
	var tooltip = this.getConfiguration()["default-description"];
	var count = params != null ? parseInt(params['private-page-size']) : 0;
	if (count > 0)
	{
		tooltip = this.getConfiguration()["private-description"];
	}
	
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(false); });
	this.getElements().each(function(element) { 
		element.updateIcon(); 
	});
	this.updateTooltip(tooltip);
	
	this._lastParams = params;
}

org.ametys.web.tool.pageaccess.control.PageAccessButton.prototype._getIcon = function(params)
{
	var nbPrivate = params != null ? parseInt(params['private-page-size']) : 0;
	var nbPublic = params != null ? parseInt(params['public-page-size']) : 0;
	
	var iconSmall = context.contextPath + this.getConfiguration()['icon-small'];
	var iconLarge = context.contextPath + this.getConfiguration()['icon-medium'];
	
	
	if (nbPrivate > 0)
	{
		iconSmall = context.contextPath + this.getConfiguration()['private-icon-small'];
		iconLarge = context.contextPath + this.getConfiguration()['private-icon-medium'];
	}

	this.getElements().each(
		function (element) 
		{
			element.iconSmall = iconSmall;
			element.iconLarge = iconLarge;
		}
	);
}
