/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.web.content');

org.ametys.web.content.ZoneMenu = function(id, configuration, pluginName) 
{
	org.ametys.web.content.ZoneMenu.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.web.content.ZoneMenu, org.ametys.cms.ribbon.button.Menu, {});

org.ametys.web.content.ZoneMenu.prototype.onMessage = function(messages)
{
	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED
			|| messages[i].getType().getName() == "templateChanged"
			|| messages[i].getType().getName() == "typeChanged")
		{
			var targets = messages[i].getTargets();
			
			var pageTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) {return target.getType() == "page"} );
			if (pageTarget != null && pageTarget.getParameters()["type"] == "container" && pageTarget.getParameters()["zone-name"] != null)
			{
				var zoneName = pageTarget.getParameters()["zone-name"];
				var zoneTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(pageTarget.getSubTargets(), function(target) {return target.getType() == "zone" && target.getParameters().id == zoneName} );
				
				if (zoneTarget != null && zoneTarget.getParameters().isModifiable != 'true')
				{
					var text = this._getTooltip(this.getConfiguration()["no-modifiable-description"]);
					this.updateTooltip(text, null, null);
					this.getElements().each(function(element) {element.disable();});
				}
				else if (this._hasRight(pageTarget))
				{
					var parent = this;
					this.getElements().each(function(button) {
						for (var itemId in button._menuItems)
						{
							var item = button._menuItems[itemId];
							item.setDisabled(!parent._hasRight(pageTarget, itemId + '-right-id'));
						}
					});
					
					if (this.getConfiguration()['selection-listener'])
					{
						eval(this.getConfiguration()['selection-listener'] + "(this, pageTarget);");
					}
					else
					{
						this.updateTooltip(this._getTooltip(), null, null);
						this.getElements().each(function(element) {element.enable();});
					}
				}
				else
				{
					this.updateTooltip(this._getTooltip(this.getConfiguration()["no-right-description"]), null, null);
					this.getElements().each(function(element) { element.disable(); });
				}
			}
			else
			{
				this.updateTooltip(this._getTooltip(this.getConfiguration()["no-selection-description"]), null, null);
				this.getElements().each(function(element) { element.disable(); });
			}
		}
	}
}

org.ametys.web.content.ZoneMenu.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.web.content.ZoneMenu.prototype.configurationChanged = function(params)
{
	if (this.getConfiguration()['configuration-changed-listener'])
	{
		eval(this.getConfiguration()['configuration-changed-listener'] + "(this, params);");
	}
}

/**
 * Checks the user right on current target
 * @return true if the user has right to do the action
 * @private
 */
org.ametys.web.content.ZoneMenu.prototype._hasRight = function(target, id)
{
	id = id || 'right-id';
	
	if (!this.getConfiguration()[id])
	{
		// No right is needed
		return true;
	}
	
	var neededRights = this.getConfiguration()[id].split('|');
	
	var rights = target.getParameters().rights;
	for (var i=0;  rights != null && i < rights.length; i++)
	{
		if (neededRights.indexOf(rights[i]) != -1)
		{
			return true;
		}
	}
	return false;
}

org.ametys.web.content.ZoneMenu.prototype._getTooltip = function (addedText)
{
	var tooltip = this.getConfiguration()["default-description"];
	if (addedText != null && addedText.length > 0)
	{
		tooltip += "<br/><br/>";
		
		tooltip += addedText;
	}
	return tooltip;
}
